// Attributes
attribute vec4 a_position;
attribute vec2 a_texCoord;
attribute vec3 a_normal;

// Uniforms
uniform mat4 u_worldViewProjectionMatrix;
uniform vec3 u_cameraPosition;
uniform mat4 u_worldMatrix;

// Varyings
varying vec2 o_vUV;
varying vec3 o_vEye;
varying vec3 o_vLight;

const vec3 LIGHT_POS = vec3(32.0, -6.0, 32.0);

void main()
{
	vec3 P = vec3(u_worldMatrix * a_position);
	vec3 N = mat3(u_worldMatrix) * a_normal;
	vec3 T = cross(N, vec3(0.0, 1.0, 0.0));
	if (length(T) == 0.0) {
		T = vec3(0.0, 0.0, 1.0);
	}
	vec3 B = cross(N, T);
	vec3 E = u_cameraPosition - P;
	o_vEye.x = dot(E, T);
	o_vEye.y = dot(E, B);
	o_vEye.z = dot(E, N);

	//vec3 L = vec3(P.x, 2.0, P.z) - P;
	vec3 L = LIGHT_POS - P;
	o_vLight.x = dot(L, T);
	o_vLight.y = dot(L, B);
	o_vLight.z = dot(L, N);

	o_vUV = a_texCoord;
	gl_Position = u_worldViewProjectionMatrix * a_position;
}
